@@ -2,6 +2,28 @@
 images or software in container images with pytest.
 
 """
+from .build import GitRepositoryBuild
+from .build import MultiStageBuild
+from .container import BindMount
+from .container import Container
+from .container import container_and_marks_from_pytest_param
+from .container import container_from_pytest_param
+from .container import container_to_pytest_param
+from .container import ContainerVolume
+from .container import DerivedContainer
+from .helpers import add_extra_run_and_build_args_options
+from .helpers import add_logging_level_options
+from .helpers import auto_container_parametrize
+from .helpers import get_extra_build_args
+from .helpers import get_extra_run_args
+from .helpers import set_logging_level_from_cli_args
+from .inspect import PortForwarding
+from .runtime import DockerRuntime
+from .runtime import get_selected_runtime
+from .runtime import OciRuntimeBase
+from .runtime import PodmanRuntime
+from .runtime import Version
+
 __all__ = [
     "GitRepositoryBuild",
     "MultiStageBuild",
@@ -22,24 +44,6 @@ __all__ = [
     "OciRuntimeBase",
     "PodmanRuntime",
     "Version",
+    "ContainerVolume",
+    "BindMount",
 ]
-
-from .build import GitRepositoryBuild
-from .build import MultiStageBuild
-from .container import Container
-from .container import container_and_marks_from_pytest_param
-from .container import container_from_pytest_param
-from .container import container_to_pytest_param
-from .container import DerivedContainer
-from .helpers import add_extra_run_and_build_args_options
-from .helpers import add_logging_level_options
-from .helpers import auto_container_parametrize
-from .helpers import get_extra_build_args
-from .helpers import get_extra_run_args
-from .helpers import set_logging_level_from_cli_args
-from .inspect import PortForwarding
-from .runtime import DockerRuntime
-from .runtime import get_selected_runtime
-from .runtime import OciRuntimeBase
-from .runtime import PodmanRuntime
-from .runtime import Version
