@@ -20,7 +20,7 @@ from dataclasses import dataclass
 from dataclasses import field
 from datetime import datetime
 from datetime import timedelta
-from hashlib import md5
+from hashlib import sha3_256
 from os.path import exists
 from os.path import isabs
 from os.path import join
@@ -39,15 +39,15 @@ from typing import Type
 from typing import Union
 from uuid import uuid4
 
+import _pytest.mark
 import deprecation
+import pytest
 import testinfra
-from _pytest.mark import Mark
-from _pytest.mark import MarkDecorator
-from _pytest.mark import ParameterSet
 from filelock import BaseFileLock
 from filelock import FileLock
-from pytest import param
 from pytest_container.helpers import get_always_pull_option
+from pytest_container.helpers import get_extra_build_args
+from pytest_container.helpers import get_extra_run_args
 from pytest_container.inspect import ContainerHealth
 from pytest_container.inspect import ContainerInspect
 from pytest_container.inspect import PortForwarding
@@ -174,9 +174,13 @@ class ContainerVolumeBase:
     #:
     #: Note that some flags are mutually exclusive and potentially not supported
     #: by all container runtimes.
+    #:
     #: The :py:attr:`VolumeFlag.SELINUX_PRIVATE` flag will be added by default
-    #: to the flags unless :py:attr:`ContainerVolumeBase.shared` is ``True``.
-    flags: List[VolumeFlag] = field(default_factory=list)
+    #: if flags is ``None``, unless :py:attr:`ContainerVolumeBase.shared` is
+    #: ``True``, then :py:attr:`VolumeFlag.SELINUX_SHARED` is added.
+    #:
+    #: If flags is a list (even an empty one), then no flags are added.
+    flags: Optional[List[VolumeFlag]] = None
 
     #: Define whether this volume should can be shared between
     #: containers. Defaults to ``False``.
@@ -190,15 +194,12 @@ class ContainerVolumeBase:
     _vol_name: str = ""
 
     def __post_init__(self) -> None:
-        if (
-            VolumeFlag.SELINUX_PRIVATE not in self.flags
-            and VolumeFlag.SELINUX_SHARED not in self.flags
-        ):
-            self.flags.append(
+        if self.flags is None:
+            self.flags = [
                 VolumeFlag.SELINUX_SHARED
                 if self.shared
                 else VolumeFlag.SELINUX_PRIVATE
-            )
+            ]
 
         for mutually_exclusive_flags in (
             (VolumeFlag.READ_ONLY, VolumeFlag.READ_WRITE),
@@ -498,9 +499,12 @@ class ContainerBase:
     _is_local: bool = False
 
     def __post_init__(self) -> None:
-        if self.url.split(":", maxsplit=1)[0] == "containers-storage":
+        local_prefix = "containers-storage:"
+        if self.url.startswith(local_prefix):
             self._is_local = True
-            self.url = self.url.replace("containers-storage:", "")
+            # returns before_separator, separator, after_separator
+            before, sep, self.url = self.url.partition(local_prefix)
+            assert before == "" and sep == local_prefix
 
     def __str__(self) -> str:
         return self.url or self.container_id
@@ -619,7 +623,12 @@ class ContainerBase:
                 all_elements.append("".join(value.values()))
             else:
                 all_elements.append(str(value))
-        return f"{md5((''.join(all_elements)).encode()).hexdigest()}.lock"
+
+        # Use a FIPS supported algorithm in here to avoid potential issues on
+        # hosts running in FIPS mode
+        # Unfortunately, we cannot use the usedforsecurity=False parameter, as
+        # that is not available on old python versions that we still support
+        return f"{sha3_256((''.join(all_elements)).encode()).hexdigest()}.lock"
 
 
 class ContainerBaseABC(ABC):
@@ -895,11 +904,21 @@ class ContainerData:
         """
         return self._container_runtime.inspect_container(self.container_id)
 
+    def read_container_logs(self) -> str:
+        """Returns the logs from the running container."""
+        return check_output(
+            [self._container_runtime.runner_binary, "logs", self.container_id]
+        ).decode()
+
 
 def container_to_pytest_param(
     container: ContainerBase,
-    marks: Optional[Union[Collection[MarkDecorator], MarkDecorator]] = None,
-) -> ParameterSet:
+    marks: Optional[
+        Union[
+            Collection[_pytest.mark.MarkDecorator], _pytest.mark.MarkDecorator
+        ]
+    ] = None,
+) -> _pytest.mark.ParameterSet:
     """Converts a subclass of :py:class:`~pytest_container.container.ContainerBase`
     (:py:class:`~pytest_container.container.Container` or
     :py:class:`~pytest_container.container.DerivedContainer`) into a
@@ -911,7 +930,7 @@ def container_to_pytest_param(
     :py:attr:`~pytest_container.container.ContainerBase.container_id`)
 
     """
-    return param(container, marks=marks or [], id=str(container))
+    return pytest.param(container, marks=marks or [], id=str(container))
 
 
 @overload
@@ -930,19 +949,21 @@ def container_and_marks_from_pytest_param(
 
 @overload
 def container_and_marks_from_pytest_param(
-    ctr_or_param: ParameterSet,
+    ctr_or_param: _pytest.mark.ParameterSet,
 ) -> Tuple[
     Union[Container, DerivedContainer],
-    Optional[Collection[Union[MarkDecorator, Mark]]],
+    Optional[Collection[Union[_pytest.mark.MarkDecorator, _pytest.mark.Mark]]],
 ]:
     ...
 
 
 def container_and_marks_from_pytest_param(
-    ctr_or_param: Union[ParameterSet, Container, DerivedContainer],
+    ctr_or_param: Union[
+        _pytest.mark.ParameterSet, Container, DerivedContainer
+    ],
 ) -> Tuple[
     Union[Container, DerivedContainer],
-    Optional[Collection[Union[MarkDecorator, Mark]]],
+    Optional[Collection[Union[_pytest.mark.MarkDecorator, _pytest.mark.Mark]]],
 ]:
     """Extracts the :py:class:`~pytest_container.container.Container` or
     :py:class:`~pytest_container.container.DerivedContainer` and the
@@ -973,7 +994,7 @@ def container_and_marks_from_pytest_param(
     details="use container_and_marks_from_pytest_param instead",
 )  # type: ignore
 def container_from_pytest_param(
-    param: Union[ParameterSet, Container, DerivedContainer],
+    param: Union[_pytest.mark.ParameterSet, Container, DerivedContainer],
 ) -> Union[Container, DerivedContainer]:
     """Extracts the :py:class:`~pytest_container.container.Container` or
     :py:class:`~pytest_container.container.DerivedContainer` from a
@@ -1031,6 +1052,27 @@ class ContainerLauncher:
         default_factory=lambda: join(tempfile.gettempdir(), str(uuid4()))
     )
 
+    @staticmethod
+    def from_pytestconfig(
+        container: Union[Container, DerivedContainer],
+        container_runtime: OciRuntimeBase,
+        pytestconfig: pytest.Config,
+        container_name: str = "",
+    ) -> "ContainerLauncher":
+        """Constructor of :py:class:`ContainerLauncher` that obtains the
+        attributes :py:attr:`rootdir`, :py:attr:`extra_build_args` and
+        :py:attr:`extra_run_args` from the pytest configuration object.
+
+        """
+        return ContainerLauncher(
+            container=container,
+            container_runtime=container_runtime,
+            rootdir=pytestconfig.rootpath,
+            extra_build_args=get_extra_build_args(pytestconfig),
+            extra_run_args=get_extra_run_args(pytestconfig),
+            container_name=container_name,
+        )
+
     def __enter__(self) -> "ContainerLauncher":
         return self
 
@@ -1053,7 +1095,14 @@ class ContainerLauncher:
         def release_lock() -> None:
             _logger.debug("Releasing lock %s", lock.lock_file)
             lock.release()
-            os.unlink(lock.lock_file)
+            # we're fine with another process/thread having deleted the
+            # lockfile, as long as the locking was thread safe
+            try:
+                # no we can't use Path.unlink(missing_ok=True) here, as the kw
+                # argument is not present in Python < 3.8
+                os.unlink(lock.lock_file)
+            except FileNotFoundError:
+                pass
 
         # Container preparation can fail, but then we would never release the
         # lock as release_lock is not yet in self._stack. However, we do not
@@ -1114,7 +1163,7 @@ class ContainerLauncher:
             _logger.debug("Launching container via: %s", launch_cmd)
             check_output(launch_cmd)
 
-        with open(self._cidfile, "r") as cidfile:
+        with open(self._cidfile, "r", encoding="utf8") as cidfile:
             self._container_id = cidfile.read(-1).strip()
 
         self._wait_for_container_to_become_healthy()
@@ -1157,13 +1206,18 @@ class ContainerLauncher:
 
         if timeout is not None and timeout > timedelta(seconds=0):
             _logger.debug(
-                "Container has a healthcheck defined, will wait at most %s ms",
-                timeout,
+                "Container has a healthcheck defined, will wait at most %s s",
+                timeout.total_seconds(),
             )
             while True:
-                health = self.container_runtime.get_container_health(
+                inspect = self.container_runtime.inspect_container(
                     self._container_id
                 )
+                if not inspect.state.running:
+                    raise RuntimeError(
+                        f"Container {self._container_id} is not running, got {inspect.state.status}"
+                    )
+                health = inspect.state.health
                 _logger.debug("Container has the health status %s", health)
 
                 if health in (
@@ -1175,8 +1229,8 @@ class ContainerLauncher:
                 if delta > timeout:
                     raise RuntimeError(
                         f"Container {self._container_id} did not become healthy within "
-                        f"{1000 * timeout.total_seconds()}ms, took {delta} and "
-                        f"state is {str(health)}"
+                        f"{timeout.total_seconds()}s, took "
+                        f"{delta.total_seconds()}s and state is {str(health)}"
                     )
                 time.sleep(max(0.5, timeout.total_seconds() / 10))
 
@@ -1191,6 +1245,19 @@ class ContainerLauncher:
             mounts = self.container_runtime.inspect_container(
                 self._container_id
             ).mounts
+
+            _logger.debug(
+                "Stopping container %s via %s",
+                self._container_id,
+                self.container_runtime.runner_binary,
+            )
+            check_output(
+                [
+                    self.container_runtime.runner_binary,
+                    "stop",
+                    self._container_id,
+                ]
+            )
             _logger.debug(
                 "Removing container %s via %s",
                 self._container_id,
